/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : S32K3XX
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 1.0.0
*   Build Version        : S32K3_RTD_1_0_0_D2110_ASR_REL_4_4_REV_0000_20211007
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

/**
 *   @file main.c
 * 主函数入口
 *   @addtogroup main_module main module documentation
 *   @{
 */

/* Including necessary configuration files. */
#include "Mcal.h"
#include "CAN.h"
#include "SchM_Can.h"
#include "Mcu.h"
#include "Mcl.h"
#include "Port.h"
#include "Dio.h"
#include "Uart.h"
#include "Eep.h"
#include "Fls.h"
#include "SchM_Fls.h"
#include "Platform.h"
#include "Lpuart_Uart_Ip_Irq.h"
#include "Flexio_Uart_Ip_Irq.h"
#include "Dma_Ip.h"
#include "Dma_Ip_Irq.h"
#include "Lpuart_Uart_Ip.h"
#include "Gpt.h"
#include "Wdg_43_Instance0.h"
#include "Icu.h"
/* User includes */

#include <string.h>
#include "hal_adapter.h"
#include "Lpuart_Uart_Ip.h"

#include "AppTaskMain.h"
#include "AppTaskUart0.h"
#include "AppTaskUart1.h"
#include "AppTaskCan.h"
#include "AppTaskGps.h"
#include "Hal_Fls.h"
#include "SEGGER_SYSVIEW.h"
#include "cm_backtrace.h"
#define APPNAME "S32K146_4G"
#define HARDWARE_VERSION "V1.0.0"
#define SOFTWARE_VERSION "V0.0.1"
int main(void)
{
	volatile int exit_code = 0;
	coreInit();
	SystemModulesInit();
	displayResetReasonWithLED();
#ifdef SEGGER_SYSTEMVIEW
	SEGGER_SYSVIEW_Conf();
	SEGGER_SYSVIEW_Start();
#endif
#ifdef SEGGER_RTT_PRINTF
	SEGGER_RTT_Init();
	cm_backtrace_init(APPNAME, HARDWARE_VERSION, SOFTWARE_VERSION);
	SEGGER_RTT_printf("boot\n");
#endif
	UartInit();
	xTaskCreate(MainTask, (const char *const)"MainTask", 2048, (void *)0, main_TASK_PRIORITY + 6, MainTask_Handle);
	//xTaskCreate(Uart0Task, (const char *const)"Uart0_Bms_Task", 512, (void *)0, main_TASK_PRIORITY + 2, Uart0Task_Handle);
	xTaskCreate(CanTask, (const char *const)"CanTask", 512, (void *)0, main_TASK_PRIORITY + 2, &CanTask_Handle);
	xTaskCreate(GpsTask, (const char *const)"GpsTask", 2048, (void *)0, main_TASK_PRIORITY + 1, &GpsTask_Handle);
	xTaskCreate(Uart_4G_Task, (const char *const)"Uart_4G_Task", 2048, (void *)0, main_TASK_PRIORITY + 0, &Uart_4G_Task_Handle);
	vTaskStartScheduler();

	for (;;)
	{
		if (exit_code != 0)
		{
			break;
		}
	}
	return exit_code;
}

/** @} */
