/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : IPV_FTFC
*   Dependencies         : 
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 1.0.0
*   Build Version        : S32K1_RTD_1_0_0_HF01_D2109_ASR_REL_4_4_REV_0000_20210907
*
*   (c) Copyright 2020-2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

/**
* @file Eep_VS_0_PBcfg.c
*
* @addtogroup EEP
* @{
*/

#ifdef __cplusplus
extern "C"{
#endif

/*==================================================================================================
*                                        INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Eep.h"


/* If FTFC IP is present on the platform */
#include "Ftfc_Eep_Ip_Cfg.h"


/*==================================================================================================
*                              SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define EEP_VS_0_VENDOR_ID_PBCFG_C                      43
#define EEP_VS_0_AR_RELEASE_MAJOR_VERSION_PBCFG_C       4
#define EEP_VS_0_AR_RELEASE_MINOR_VERSION_PBCFG_C       4
#define EEP_VS_0_AR_RELEASE_REVISION_VERSION_PBCFG_C    0
#define EEP_VS_0_SW_MAJOR_VERSION_PBCFG_C               1
#define EEP_VS_0_SW_MINOR_VERSION_PBCFG_C               0
#define EEP_VS_0_SW_PATCH_VERSION_PBCFG_C               0

/*==================================================================================================
*                                     FILE VERSION CHECKS
==================================================================================================*/
/* Check if current file and Eep header file are of the same vendor */
#if (EEP_VS_0_VENDOR_ID_PBCFG_C != EEP_VENDOR_ID)
    #error "AutoSar Version Numbers of Eep_VS_0_PBcfg.c and Eep.h have different vendor ids"
#endif
/* Check if current file and Eep header file are of the same Autosar version */
#if ((EEP_VS_0_AR_RELEASE_MAJOR_VERSION_PBCFG_C    != EEP_AR_RELEASE_MAJOR_VERSION) || \
     (EEP_VS_0_AR_RELEASE_MINOR_VERSION_PBCFG_C    != EEP_AR_RELEASE_MINOR_VERSION) || \
     (EEP_VS_0_AR_RELEASE_REVISION_VERSION_PBCFG_C != EEP_AR_RELEASE_REVISION_VERSION) \
    )
    #error "AutoSar Version Numbers of Eep_VS_0_PBcfg.c and Eep.h are different"
#endif
/* Check if current file and Eep header file are of the same Software version */
#if ((EEP_VS_0_SW_MAJOR_VERSION_PBCFG_C != EEP_SW_MAJOR_VERSION) || \
     (EEP_VS_0_SW_MINOR_VERSION_PBCFG_C != EEP_SW_MINOR_VERSION) || \
     (EEP_VS_0_SW_PATCH_VERSION_PBCFG_C != EEP_SW_PATCH_VERSION) \
    )
    #error "Software Version Numbers of Eep_VS_0_PBcfg.c and Eep.h are different"
#endif


/* If SD Emmc IP is present on the platform */

/* Check if current file and Ftfc_Eep_Ip_Cfg header file are of the same vendor */
#if (EEP_VS_0_VENDOR_ID_PBCFG_C != FTFC_EEP_IP_CFG_VENDOR_ID_H)
    #error "AutoSar Version Numbers of Eep_VS_0_PBcfg.c and Ftfc_Eep_Ip_Cfg.h have different vendor ids"
#endif
/* Check if current file and Ftfc_Eep_Ip_Cfg header file are of the same Autosar version */
#if ((EEP_VS_0_AR_RELEASE_MAJOR_VERSION_PBCFG_C    != FTFC_EEP_IP_CFG_AR_RELEASE_MAJOR_VERSION_H) || \
     (EEP_VS_0_AR_RELEASE_MINOR_VERSION_PBCFG_C    != FTFC_EEP_IP_CFG_AR_RELEASE_MINOR_VERSION_H) || \
     (EEP_VS_0_AR_RELEASE_REVISION_VERSION_PBCFG_C != FTFC_EEP_IP_CFG_AR_RELEASE_REVISION_VERSION_H) \
    )
    #error "AutoSar Version Numbers of Eep_VS_0_PBcfg.c and Ftfc_Eep_Ip_Cfg.h are different"
#endif
/* Check if current file and Ftfc_Eep_Ip_Cfg header file are of the same Software version */
#if ((EEP_VS_0_SW_MAJOR_VERSION_PBCFG_C != FTFC_EEP_IP_CFG_SW_MAJOR_VERSION_H) || \
     (EEP_VS_0_SW_MINOR_VERSION_PBCFG_C != FTFC_EEP_IP_CFG_SW_MINOR_VERSION_H) || \
     (EEP_VS_0_SW_PATCH_VERSION_PBCFG_C != FTFC_EEP_IP_CFG_SW_PATCH_VERSION_H) \
    )
    #error "Software Version Numbers of Eep_VS_0_PBcfg.c and Ftfc_Eep_Ip_Cfg.h are different"
#endif




/*==================================================================================================
                                     FUNCTION PROTOTYPES
==================================================================================================*/

/*==================================================================================================
                                           CONSTANTS
==================================================================================================*/
#define EEP_START_SEC_CONFIG_DATA_UNSPECIFIED
#include "Eep_MemMap.h"

/**
* @brief       Structure used to set function pointers notification, working mode
*/
const Eep_ConfigType Eep_Config_VS_0 =
{
    /* EepJobEndNotification             */ NULL_PTR,
    /* EepJobErrorNotification           */ NULL_PTR,
    /* EepDefaultMode                    */ MEMIF_MODE_SLOW,
    /* EepFastReadBlockSize              */ 4096U,
    /* EepNormalReadBlockSize            */ 4096U,
    /* EepFastWriteBlockSize             */ 4096U,
    /* EepNormalWriteBlockSize           */ 4096U,
    /* EEP Config Set CRC checksum       */ 16356U,
    /* FTFC IP Config Set                */ &EepFtfcCfg_VS_0};

#define EEP_STOP_SEC_CONFIG_DATA_UNSPECIFIED
#include "Eep_MemMap.h"

#ifdef __cplusplus
}
#endif

/** @}*/
